"""
Module: '_espnow' on micropython-v1.27.0-esp32-ESP32_GENERIC
"""

# MCU: {'variant': '', 'build': '', 'arch': 'xtensawin', 'port': 'esp32', 'board': 'ESP32_GENERIC', 'board_id': 'ESP32_GENERIC', 'mpy': 'v6.3', 'ver': '1.27.0', 'family': 'micropython', 'cpu': 'ESP32', 'version': '1.27.0'}
# Stubber: v1.26.4
from __future__ import annotations
from typing import Final
from _typeshed import Incomplete

RATE_LORA_500K: Final[int] = 42
RATE_2M: Final[int] = 1
RATE_24M: Final[int] = 9
RATE_1M: Final[int] = 0
RATE_54M: Final[int] = 12
RATE_LORA_250K: Final[int] = 41
RATE_6M: Final[int] = 11
RATE_5M: Final[int] = 2
KEY_LEN: Final[int] = 16
MAX_DATA_LEN: Final[int] = 250
ADDR_LEN: Final[int] = 6
RATE_12M: Final[int] = 10
MAX_ENCRYPT_PEER_NUM: Final[int] = 6
RATE_11M: Final[int] = 3
MAX_TOTAL_PEER_NUM: Final[int] = 20

class ESPNowBase:
    def irq(self, *args, **kwargs) -> Incomplete: ...
    def mod_peer(self, *args, **kwargs) -> Incomplete: ...
    def get_peers(self, *args, **kwargs) -> Incomplete: ...
    def stats(self, *args, **kwargs) -> Incomplete: ...
    def set_pmk(self, *args, **kwargs) -> Incomplete: ...
    def peer_count(self, *args, **kwargs) -> Incomplete: ...
    def recvinto(self, *args, **kwargs) -> Incomplete: ...
    def send(self, *args, **kwargs) -> Incomplete: ...
    def active(self, *args, **kwargs) -> Incomplete: ...
    def any(self, *args, **kwargs) -> Incomplete: ...
    def get_peer(self, *args, **kwargs) -> Incomplete: ...
    def del_peer(self, *args, **kwargs) -> Incomplete: ...
    def add_peer(self, *args, **kwargs) -> Incomplete: ...
    def config(self, *args, **kwargs) -> Incomplete: ...
    def __init__(self, *argv, **kwargs) -> None: ...
